#include <platform.h>
#include <osw.h>
#include <trunksys.h>
#include <motfreq.h>

static char myres[14];
/*
   Input Range (Hex)     800 Plan Range         Splinter Plan Range
   000 - 2cf             851.0125 - 868.9875    851.0000 - 868.9750 * pivot = 257 = 865.9875
   2d0 - 2f7             866.0000 - 866.9750    866.0000 - 866.9750 * same * split *
   32f - 33f             867.0125 - 867.4000    867.0000 - 867.4000 * same * split *
   3c1 - 3fe             867.4250 - 868.9500    867.4250 - 868.9500 * same * split *
   3be -                 867.9750               867.9750            * same * split *

c v856.2250,0d1,ffff   - matches splinter 
  v856.4250,0d9,ffff   - matches splinter
  v858.2250,121,ffff   - matches splinter
  v866.4875,26b,ffff   * matches NORMAL
  v867.4875,293,ffff   * matches NORMAL
 dv867.9875,2a7,ffff   * matches NORMAL
 dv868.4875,2bb,ffff

*/


static inline char *
map800CodeToF(unsigned short theCode)   /* trunker code '8' */
{
	if( theCode <= 0x2cf ) {
		sprintf(myres,"%8.4f",851.0125 + (0.025 * ((double) theCode)));
	} else if( theCode <= 0x2f7 ) {
		sprintf(myres,"%8.4f",866.0000 + (0.025 * ((double) (theCode-0x2d0))));
	} else if( theCode >= 0x32f && theCode <= 0x33f ) {
		sprintf(myres,"%8.4f",867.0000 + (0.025 * ((double) (theCode-0x32f))));
	} else if( theCode >= 0x3c1 && theCode <= 0x3fe ) {
		sprintf(myres,"%8.4f",867.4250 + (0.025 * ((double) (theCode-0x3c1))));
	} else if( theCode == 0x3BE ) {
		return "868.9750";
	} else{
		sprintf(myres,"0x%03hx   ",(unsigned short)theCode); // unknown code
	}
	return myres;
}
static inline char *  /* trunker code 'S' */
map800SCodeToF(unsigned short theCode) // this is for systems that are usually off by .0125mhz
{
	if( theCode <= 0x257 ) {
		sprintf(myres,"%8.4f",851.0000 + (0.025 * ((double) theCode)));
	} else if( theCode <= 0x2cf ) {
		sprintf(myres,"%8.4f",866.0125 + (0.025 * ((double) (theCode-0x258))));
	} else if( theCode <= 0x2f7 ) {
		sprintf(myres,"%8.4f",866.0000 + (0.025 * ((double) (theCode-0x2d0))));
	} else if( theCode >= 0x32f && theCode <= 0x33f ) {
		sprintf(myres,"%8.4f",867.0000 + (0.025 * ((double) (theCode-0x32f))));
	} else if( theCode >= 0x3c1 && theCode <= 0x3fe ) {
		sprintf(myres,"%8.4f",867.4250 + (0.025 * ((double) (theCode-0x3c1))));
	} else if( theCode == 0x3BE ) {
		return "868.9750";
	} else{
		sprintf(myres,"0x%03hx   ",(unsigned short)theCode); // unknown code
	}
	return myres;
}
static inline char *
map900CodeToF(unsigned short theCode)
{
	sprintf(myres,"%8.4f",935.0125 + (0.0125 * ((double) theCode)));
	return myres;
}
char *
MotorolaMap::mapCodeToF(unsigned short theCode, char bp)
{
	switch(bp)
	{
case '8': return map800CodeToF(theCode);
case 'S': return map800SCodeToF(theCode);
case '9': return map900CodeToF(theCode);
 default: return mapOtherCodeToF(theCode);
	}
}
BOOL
MotorolaMap::isFrequency(unsigned short theCode, char bp) {
	if( bp == '9' ) {
		return (theCode >= 0 && theCode <= 0x1DE ) ? TRUE : FALSE;			
	} else { // all others - fairly permissive - covers any schemes I've heard of...
			 // doesn't interfere with any of the OSWs I know of... consistent with
			 // data sent to me anonymously....
		return ((theCode >= 0 && theCode <= 0x2F7 )
		      ||(theCode >= 0x32f && theCode <= 0x33F)
		      ||(theCode >= 0x3c1 && theCode <= 0x3FE)
		      ||(theCode == 0x3BE)) ? TRUE : FALSE;
	}
}
